package envoy.client.data;

import java.util.function.Function;

/**
 * Contains a single {@link Config} value as well as the corresponding command
 * line arguments and its default value.<br>
 * <br>
 * Project: <strong>envoy-clientChess</strong><br>
 * File: <strong>ConfigItem.javaEvent.java</strong><br>
 * Created: <strong>21.12.2019</strong><br>
 *
 * @author Kai S. K. Engelbart
 * @param <T> the type of the config item's value
 * @since Envoy v0.2-alpha
 */
public class ConfigItem<T> {

	private final String				commandLong, commandShort;
	private final Function<String, T>	parseFunction;
	private final boolean				mandatory;

	private T value;

	/**
	 * Initializes a {@link ConfigItem}.
	 *
	 * @param commandLong   the long command line argument to set this value
	 * @param commandShort  the short command line argument to set this value
	 * @param parseFunction the {@code Function<String, T>} that parses the value
	 *                      from a string
	 * @param defaultValue  the optional default value to set before parsing
	 * @param mandatory     indicated that this config item must be initialized with
	 *                      a non-null value
	 * @since Envoy v0.2-alpha
	 */
	public ConfigItem(String commandLong, String commandShort, Function<String, T> parseFunction, T defaultValue, boolean mandatory) {
		this.commandLong	= commandLong;
		this.commandShort	= commandShort;
		this.parseFunction	= parseFunction;
		this.mandatory		= mandatory;
		value				= defaultValue;
	}

	/**
	 * Initializes an optional {@link ConfigItem} without a default value.
	 *
	 * @param commandLong   the long command line argument to set this value
	 * @param commandShort  the short command line argument to set this value
	 * @param parseFunction the {@code Function<String, T>} that parses the value
	 *                      from a string
	 * @since Envoy v0.3-alpha
	 */
	public ConfigItem(String commandLong, String commandShort, Function<String, T> parseFunction) {
		this(commandLong, commandShort, parseFunction, null, false);
	}

	/**
	 * Parses this {@ConfigItem}'s value from a string.
	 *
	 * @param input the string to parse from
	 * @since Envoy v0.2-alpha
	 */
	public void parse(String input) { value = parseFunction.apply(input); }

	/**
	 * @return The long command line argument to set the value of this
	 *         {@link ConfigItem}
	 * @since Envoy v0.2-alpha
	 */
	public String getCommandLong() { return commandLong; }

	/**
	 * @return The short command line argument to set the value of this
	 *         {@link ConfigItem}
	 * @since Envoy v0.2-alpha
	 */
	public String getCommandShort() { return commandShort; }

	/**
	 * @return the value of this {@link ConfigItem}
	 * @since Envoy v0.2-alpha
	 */
	public T get() { return value; }

	/**
	 * @return {@code true} if this {@link ConfigItem} is mandatory for successful
	 *         application initialization
	 * @since Envoy v0.3-alpha
	 */
	public boolean isMandatory() { return mandatory; }
}
